# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os
import tempfile

from . import indent
from . import lint
from . import style
from . import utils
from .path import PathChecker
from .utils import vcs

TMP_FILE_DIR = tempfile.gettempdir()
LOG_FILE_PATH = os.path.join(TMP_FILE_DIR, 'check-patch.log')

# Rely on built-in recursion limit to limit number of directories searched


class FileChecker(object):

    """
    Picks up a given file and performs various checks, looking after problems
    and eventually applying solutions when all possible.
    """

    def __init__(self, args):
        """
        Class constructor, sets the file checkers.

        :param path: Path to the file that will be checked.
        :param vcs: Version control system being used.
        :param confirm: Whether to answer yes to all questions asked without
                prompting the user.
        """
        assert args.disable is not None
        assert args.pep8_disable is not None
        self.args = args
        self.linter = lint.Linter(self.args)
        self.indenter = indent.Reindenter(self.args)
        # Tweak --disable option for StyleChecker
        self.args.disable = self.args.pep8_disable
        self.style_checker = style.StyleChecker(self.args)
        self.vcs = vcs.VCS()

    def _check_indent(self, path):
        """
        Verifies the file with the reindent module.

        This tool performs the following checks on python files:

          * Trailing whitespaces
          * Tabs
          * End of line
          * Incorrect indentation

        And will automatically correct problems.
        """
        return self.indenter.check_file(path)

    def _check_syntax(self, path):
        """
        Verifies the file with pylint.
        """
        return self.linter.check_file(path)

    def _check_style(self, path):
        """
        Verifies the file compliance to PEP8.
        """
        return self.style_checker.check_file(path)

    def _check_permissions(self, path):
        """
        Verifies the execution permissions and fixes them if possible:

          * Files with no shebang and execution permissions will be fixed.
          * Files with shebang and no execution permissions will be fixed.
        """
        result = True
        if os.path.isdir(path):
            return result
        checker = PathChecker(path, self.args)
        path_is_script = checker.path.script
        path_is_exec = checker.path.executable
        if path_is_script:
            if not path_is_exec:
                result = False
                self.vcs.set_file_executable(path)
        else:
            if path_is_exec:
                result = False
                self.vcs.unset_file_executable(path)
        return result

    def check_file(self, path):
        """
        Executes all required checks, if problems are found, the possible
        corrective actions are listed.
        """
        return (self._check_syntax(path) and
                self._check_indent(path) and
                self._check_style(path) and
                self._check_permissions(path))


class PatchChecker(FileChecker):

    def __init__(self, args, patch=None,
                 logger=logging.getLogger('')):
        super(PatchChecker, self).__init__(args)
        self.base_dir = TMP_FILE_DIR
        self.log = logger
        self.patch = patch
        self.untracked_files = None

        if patch:
            self.patch = os.path.abspath(patch)

    def validate(self):
        if not os.path.isfile(self.patch):
            logging.error("Invalid patch file %s provided. Aborting.",
                          self.patch)
            return 1

        changed_files_before = self.vcs.get_modified_files()
        if changed_files_before:
            self.log.error("Repository has changed files prior to patch "
                           "application")
            answer = utils.ask("Would you like to revert them?")
            if answer == "n":
                self.log.error("Not safe to proceed without reverting files.")
                return 1
            else:
                for changed_file in changed_files_before:
                    self.vcs.revert_file(changed_file)

        self.untracked_files = self.vcs.get_unknown_files()

    def check_modified_files(self):
        files_failed_check = []

        modified_files = self.vcs.get_modified_files_patch(self.untracked_files,
                                                           self.patch)
        for modified_file in modified_files:
            # Additional safety check, new commits might introduce
            # new directories
            if os.path.isfile(modified_file):
                if not self.check_file(modified_file):
                    files_failed_check.append(modified_file)

        if files_failed_check:
            return False
        else:
            return True

    def check(self):
        self.vcs.apply_patch(self.patch)
        passed_check = self.check_modified_files()
        if passed_check:
            self.log.info('Patch %s check PASS', self.patch)
            return 0
        else:
            self.log.error('Patch %s check FAIL', self.patch)
            return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import fnmatch
import os
import stat
import logging
import codecs
from inspektor.utils.data_structures import Borg

PY_EXTENSIONS = ['.py']
SHEBANG = '#!'


class PathRegistry(Borg):

    def __init__(self):
        Borg.__init__(self)
        if not getattr(self, 'registry', None):
            self.registry = {}

    def get(self, path):
        return self.registry.get(path)

    def set(self, path_attribute):
        self.registry[path_attribute.path] = path_attribute


class PathAttribute(object):
    def __init__(self, path, ignore_patterns):
        path_registry = PathRegistry()
        cached_path_attribute = path_registry.get(path=path)
        if cached_path_attribute:
            self.__dict__ = cached_path_attribute.__dict__
        else:
            self._path = path
            self._ignore_patterns = ignore_patterns
            self._ignore = None
            self._text = None
            self._first_line = None
            self._executable = None
            self._empty = None
            self._python = None
            path_registry.set(self)

    def __str__(self):
        return self._path

    @property
    def path(self):
        return self._path

    @property
    def text(self):
        if self._text is not None:
            return self._text
        if self.ignore or self.first_line is None:
            self._text = False
            return self._text
        else:
            self._text = True
            return self._text

    @property
    def binary(self):
        return not self.text

    @property
    def first_line(self):
        if self._first_line is not None:
            return self._first_line

        if os.path.isfile(self._path):
            try:
                with codecs.open(self._path, 'r', encoding='utf-8') as checked_file:
                    self._first_line = checked_file.readline()
                    return self._first_line
            except UnicodeDecodeError:
                self._ignore = True
                self._text = False

    @property
    def executable(self):
        if self._executable is not None:
            return self._executable
        mode = os.stat(self._path)[stat.ST_MODE]
        self._executable = mode & stat.S_IXUSR
        return self._executable

    @property
    def not_empty(self):
        return not self.empty

    @property
    def empty(self):
        if self._empty is not None:
            return self._empty
        size = os.stat(self._path)[stat.ST_SIZE]
        self._empty = size == 0
        return self._empty

    def script(self, language=None):
        if self.first_line:
            if self.first_line.startswith(SHEBANG):
                if language is None:
                    return True
                elif language in self.first_line:
                    return True
        return False

    @property
    def python(self):
        if self._python is not None:
            return self._python
        for extension in PY_EXTENSIONS:
            if self._path.endswith(extension):
                self._python = True
                return self._python

        return self.script(language='python')

    @property
    def ignore(self):
        if self._ignore is not None:
            return self._ignore
        else:
            path = self._path
            if path.startswith('./'):
                path = path[2:]
            for pattern in self._ignore_patterns:
                if fnmatch.fnmatch(path, pattern):
                    self._ignore = True
                if self._path.startswith(os.path.abspath(pattern)):
                    self._ignore = True
            if self._ignore is None:
                self._ignore = False
            return self._ignore


class PathChecker(object):

    def __init__(self, path, args, label=None, logger=logging.getLogger('')):
        self.args = args
        self.ignore = ['*~', '*#', '*.swp', '*.py?', '*.o', '.git', '.svn']
        if self.args.exclude:
            self.ignore += self.args.exclude.split(',')
        self._read_gitignore()
        self.path = PathAttribute(path, ignore_patterns=self.ignore)
        if label is None:
            label = 'Check'
        self.label = label
        self.log = logger

    def _read_gitignore(self):
        config_ignore = '{home}/.config/git/ignore'.format(
            home=os.environ.get('HOME'))
        git_dir_ignore = '{gitdir}/.config/git/ignore'.format(
            gitdir=os.environ.get('GIT_DIR'))
        dot_ignore = '.gitignore'
        for ign in (dot_ignore, git_dir_ignore, config_ignore):
            if os.path.isfile(ign):
                with open(ign) as f:
                    self.ignore += [x.strip() for x in f.readlines()]
                break
        self.ignore = list(set(self.ignore))

    def check_attributes(self, *args):
        if self.path.ignore:
            return False
        checks_passed = True
        for arg in args:
            checks_passed &= getattr(self.path, arg)
        return checks_passed

    def log_status(self, status, extra=''):
        if extra:
            extra = '(%s)' % extra
        if status == 'PASS':
            self.log.debug('%s: %s %s %s', self.label, self.path, status, extra)
        elif status == 'FAIL':
            self.log.error('%s: %s %s %s', self.label, self.path, status, extra)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.


class Borg:

    """
    Multiple instances of this class will share the same state.

    This is considered a better design pattern in Python than
    more popular patterns, such as the Singleton. Inspired by
    Alex Martelli's article mentioned below:

    :see: http://www.aleax.it/5ep.html
    """

    __shared_state = {}

    def __init__(self):
        self.__dict__ = self.__shared_state



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os

from .path import PathChecker


LICENSE_SNIPPET_GPLV2 = """# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
"""

LICENSE_SNIPPET_GPLV2_STRICT = """# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; specifically version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
"""

license_mapping = {'gplv2_later': LICENSE_SNIPPET_GPLV2,
                   'gplv2_strict': LICENSE_SNIPPET_GPLV2_STRICT}
default_license = 'gplv2_later'


class LicenseChecker(object):

    def __init__(self, args, logger=logging.getLogger('')):
        self.license_type = args.license
        cpyright = args.copyright
        author = args.author
        self.args = args
        self.failed_paths = []
        self.log = logger
        self.license_contents = license_mapping[self.license_type]
        self.base_license_contents = self.license_contents

        if cpyright:
            self.license_contents += "#\n"
            self.license_contents += "# " + cpyright + "\n"

        if author:
            self.license_contents += "# " + author + "\n"

    def check_dir(self, path):
        for root, dirs, files in os.walk(path):
            for filename in files:
                self.check_file(os.path.join(root, filename))
        return not self.failed_paths

    def check_file(self, path):
        checker = PathChecker(path=path, args=self.args, label='License',
                              logger=self.log)
        # Don't put license info in empty __init__.py files.
        if not checker.check_attributes('text', 'python', 'not_empty'):
            return True

        first_line = None
        if checker.path.script('python'):
            first_line = checker.path.first_line

        with open(path, 'r') as inspected_file:
            content = inspected_file.readlines()
            if first_line is not None:
                content = content[1:]
            content = "".join(content)
            if self.base_license_contents not in content:
                fix_status = ''
                if self.args.fix:
                    new_content = ""
                    if first_line is not None:
                        new_content += first_line
                        new_content += '\n'
                    new_content += self.license_contents + '\n' + content
                    with open(path, 'w') as inspected_file:
                        inspected_file.write(new_content)
                        fix_status = 'FIX OK'

                self.failed_paths.append(path)
                checker.log_status(status='FAIL', extra=fix_status)
                return False
            else:
                checker.log_status(status='PASS')
                return True

    def check(self, path):
        if os.path.isfile(path):
            return self.check_file(path)
        elif os.path.isdir(path):
            return self.check_dir(path)
        else:
            self.log.error("Invalid location '%s'", path)
            return False



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

import logging
import os

from cliff.command import Command

from inspektor.style import StyleChecker


class StyleCommand(Command):
    """
    check against style guide with pycodestyle
    """
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(StyleCommand, self).get_parser(prog_name)
        parser.add_argument('path', type=str,
                            help='Path to check (empty for full tree check)',
                            nargs='*',
                            default=None)
        parser.add_argument('--disable', type=str,
                            help='Disable the PEP8 errors given as arguments. '
                                 'Default: %(default)s',
                            default='E501,E265,W601,E402')
        parser.add_argument('--fix', action='store_true', default=False,
                            help='Fix any style problems found '
                                 '(needs autopep8 installed)')
        parser.add_argument('--max-line-length', type=int, default=79,
                            help=('set maximum allowed line length. Default: '
                                  '%(default)s'))
        parser.add_argument('--exclude', type=str,
                            help='Quoted string containing paths or '
                                 'patterns to be excluded from '
                                 'checking, comma separated')
        return parser

    def take_action(self, parsed_args):
        paths = parsed_args.path
        if not paths:
            paths = [os.getcwd()]

        style_checker = StyleChecker(parsed_args, logger=self.log)

        status = True
        for path in paths:
            status &= style_checker.check(path)
        if status:
            self.log.info("PEP8 compliance check PASS")
            return 0
        else:
            self.log.error("PEP8 compliance FAIL")
            return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os
import sys

from pylint.lint import Run, PyLinter

from .path import PathChecker
from .utils import process

_PYLINT_HELP_TEXT = process.run('pylint --help', verbose=False).stdout


class QuietPyLinter(PyLinter):
    def read_config_file(self, config_file=None):
        quiet = self.quiet
        try:
            self.quiet = 1
            return super(QuietPyLinter, self).read_config_file()
        finally:
            self.quiet = quiet


class QuietLintRun(Run):
    LinterClass = QuietPyLinter


class Linter(object):

    def __init__(self, args, logger=logging.getLogger('')):
        self.ignored_errors = ''
        if hasattr(args, 'disable'):
            self.ignored_errors = args.disable
        elif hasattr(args, 'disable_lint'):
            self.ignored_errors = args.disable_lint
        self.enabled_errors = ''
        if hasattr(args, 'enable'):
            self.enabled_errors = args.enable
        elif hasattr(args, 'enable_lint'):
            self.enabled_errors = args.enable_lint
        self.log = logger
        self.args = args
        if hasattr(args, 'parallel'):
            self.parallel = args.parallel
        else:
            import multiprocessing
            self.parallel = multiprocessing.cpu_count()
        # Be able to analyze all imports inside the project
        sys.path.insert(0, os.getcwd())
        self.log.info('Pylint disabled: %s', self.ignored_errors)
        self.log.info('Pylint enabled : %s', self.enabled_errors)

    @staticmethod
    def _pylint_has_option(option):
        return option in _PYLINT_HELP_TEXT

    def get_opts(self):
        """
        If VERBOSE is set, show pylint reports. If not, only an issue summary.
        """
        pylint_args = ['--rcfile=/dev/null',
                       '--good-names=i,j,k,Run,_,vm',
                       ('--msg-template='
                        '"{msg_id}:{line:3d},{column}: {obj}: {msg}"')]

        if self.ignored_errors:
            pylint_args.append('--disable=%s' % self.ignored_errors)
        if self.enabled_errors:
            pylint_args.append('--enable=%s' % self.enabled_errors)
        if self._pylint_has_option('--reports='):
            pylint_args.append('--reports=no')
        if self._pylint_has_option('--score='):
            pylint_args.append('--score=no')
        if self._pylint_has_option('--jobs='):
            pylint_args.append('--jobs=%s' % self.parallel)

        return pylint_args

    def check(self, file_or_dirs):
        def should_be_checked(path):
            checker = PathChecker(path=path, args=self.args, label='Lint',
                                  logger=self.log)
            return checker.check_attributes('text', 'python', 'not_empty')
        paths = []
        not_file_or_dirs = []
        for file_or_dir in file_or_dirs:
            if os.path.isdir(file_or_dir):
                for root, _, files in os.walk(file_or_dir):
                    for filename in files:
                        path = os.path.join(root, filename)
                        if should_be_checked(path):
                            paths.append(path)
            elif os.path.isfile(file_or_dir):
                if should_be_checked(file_or_dir):
                    paths.append(file_or_dir)
            else:
                not_file_or_dirs.append(file_or_dir)
        linter_failed = True
        if paths:
            runner = QuietLintRun(self.get_opts() + paths, exit=False)
            for module, status in runner.linter.stats.get('by_module').items():
                if any(status.values()):
                    self.log.debug('Lint: %s FAIL', module)
                else:
                    self.log.debug('Lint: %s PASS', module)
            if runner.linter.msg_status == 0:
                linter_failed = False
        if not_file_or_dirs:
            self.log.error("Following arguments are not files nor dirs: %s",
                           ", ".join(not_file_or_dirs))
            return 0
        if linter_failed:
            return 0
        return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

import logging
import os

from cliff.command import Command

from inspektor.license import LicenseChecker
from inspektor.license import default_license
from inspektor.license import license_mapping


class LicenseCommand(Command):
    """
    check for license headers
    """
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(LicenseCommand, self).get_parser(prog_name)
        parser.add_argument('path', type=str,
                            help='Path to check (empty for full tree check)',
                            nargs='?',
                            default="")
        parser.add_argument('--license', type=str,
                            help=('License type. Supported license types: %s. '
                                  'Default: %s' %
                                  (license_mapping.keys(), default_license)),
                            default="gplv2_later")
        parser.add_argument('--copyright', type=str,
                            help='Copyright string. Ex: "Copyright (c) 2013-2014 FooCorp"',
                            default="")
        parser.add_argument('--author', type=str,
                            help='Author string. Ex: "Author: Brandon Lindon <brandon.lindon@foocorp.com>"',
                            default="")
        parser.add_argument('--exclude', type=str,
                            help='Quoted string containing paths or '
                                 'patterns to be excluded from '
                                 'checking, comma separated')
        parser.add_argument('--fix', action='store_true', default=False,
                            help='Fix any style problems found '
                                 '(needs autopep8 installed)')
        return parser

    def take_action(self, parsed_args):
        path = parsed_args.path

        if not path:
            path = os.getcwd()

        checker = LicenseChecker(parsed_args)

        if checker.check(path):
            self.log.info("License check PASS")
            return 0
        else:
            self.log.error("License check FAIL")
            return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os
import sys

import pycodestyle

try:
    import autopep8
    AUTOPEP8_CAPABLE = True
    del(autopep8)
except ImportError:
    AUTOPEP8_CAPABLE = False

from .path import PathChecker
from .utils import stacktrace
from .utils import process


class StyleChecker(object):

    def __init__(self, args, logger=logging.getLogger('')):
        # Be able to analyze all imports inside the project
        sys.path.insert(0, os.getcwd())
        self.log = logger
        self.failed_paths = []
        self.ignored_errors = ''
        if hasattr(args, 'disable'):
            self.ignored_errors = args.disable
        elif hasattr(args, 'disable_style'):
            self.ignored_errors = args.disable_style
        self.args = args
        self.log.info('PEP8 disabled: %s', self.ignored_errors)

    def check_dir(self, path):
        """
        Recursively go on a directory checking files with PEP8.

        :param path: Path to a directory.
        """
        for root, dirs, files in os.walk(path):
            for filename in files:
                self.check_file(os.path.join(root, filename))
        return not self.failed_paths

    def check_file(self, path):
        """
        Check one regular file with pylint for py syntax errors.

        :param path: Path to a regular file.
        :return: False, if pylint found syntax problems, True, if pylint didn't
                 find problems, or path is not a python module or script.
        """
        checker = PathChecker(path=path, args=self.args, label='Style',
                              logger=self.log)
        if not checker.check_attributes('text', 'python', 'not_empty'):
            return True

        try:
            opt_obj = pycodestyle.StyleGuide().options
            ignore_list = self.ignored_errors.split(',') + list(opt_obj.ignore)
            opt_obj.ignore = tuple(set(ignore_list))
            # pylint: disable=E1123
            runner = pycodestyle.Checker(filename=path, options=opt_obj)
            status = runner.check_all()
        except Exception:
            self.log.error('Unexpected exception while checking %s', path)
            exc_info = sys.exc_info()
            stacktrace.log_exc_info(exc_info, 'inspektor.style')
            status = 1

        if status != 0:
            self.failed_paths.append(path)
            fix_status = ''
            if AUTOPEP8_CAPABLE:
                if self.args.fix:
                    self.log.info('Trying to fix errors with autopep8')
                    try:
                        process.run('autopep8 --in-place --max-line-length=%s --ignore %s %s' % (self.args.max_line_length, self.ignored_errors, path))
                        fix_status = 'FIX OK'
                    except Exception:
                        self.log.error('Unable to fix errors')
                        exc_info = sys.exc_info()
                        stacktrace.log_exc_info(exc_info, 'inspektor.style')
                        fix_status = 'FIX NOT OK'
            else:
                self.log.error('Python library autopep8 not installed. '
                               'Please install it if you want to use --fix')
                fix_status = 'FIX NOT OK'
            checker.log_status(status='FAIL', extra=fix_status)
        else:
            checker.log_status(status='PASS')

        return status == 0

    def check(self, path):
        if os.path.isfile(path):
            return self.check_file(path)
        elif os.path.isdir(path):
            return self.check_dir(path)
        else:
            self.log.error("Invalid location '%s'", path)
            return False



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

import logging
import multiprocessing
import os

from cliff.command import Command

from inspektor.lint import Linter


class LintCommand(Command):
    """
    check for errors with pylint
    """
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(LintCommand, self).get_parser(prog_name)
        parser.add_argument('path', type=str,
                            help='Path to check (empty for full tree check)',
                            nargs='*',
                            default=None)
        parser.add_argument('--disable', type=str,
                            help='Disable the pylint errors. Default: %(default)s',
                            default='W,R,C,E1002,E1101,E1103,E1120,F0401,I0011')
        parser.add_argument('--enable', type=str,
                            help=('Enable the pylint errors '
                                  '(takes place after disabled items are '
                                  'processed). Default: %(default)s'),
                            default='W0611')
        parser.add_argument('--exclude', type=str,
                            help='Quoted string containing paths or '
                                 'patterns to be excluded from '
                                 'checking, comma separated')
        parser.add_argument('--parallel', action='store', nargs='?',
                            default=multiprocessing.cpu_count(),
                            help="How many threads to use")
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.path:
            parsed_args.path = [os.getcwd()]

        linter = Linter(parsed_args, logger=self.log)
        status = linter.check(parsed_args.path)

        if status:
            self.log.info("Syntax check PASS")
            return 0
        else:
            self.log.error("Syntax check FAIL")
            return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

import logging
import os

from cliff.command import Command

from inspektor.indent import Reindenter


class IndentCommand(Command):
    """
    check for correct file indentation
    """
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(IndentCommand, self).get_parser(prog_name)
        parser.add_argument('path', type=str,
                            help='Path to check (empty for full tree check)',
                            nargs='*',
                            default=None)
        parser.add_argument('--fix', action='store_true', default=False,
                            help='Fix any indentation problems found')
        parser.add_argument('--exclude', type=str,
                            help='Quoted string containing paths or '
                                 'patterns to be excluded from '
                                 'checking, comma separated')
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.path:
            parsed_args.path = [os.getcwd()]

        reindenter = Reindenter(parsed_args, logger=self.log)

        status = True
        for path in parsed_args.path:
            status &= reindenter.check(path)
        if status:
            self.log.info('Indentation check PASS')
            return 0
        else:
            self.log.error('Syntax check FAIL')
            return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

import logging
import os

from cliff.command import Command

from inspektor import utils
from inspektor.patch import PatchChecker


class GithubPatchChecker(PatchChecker):

    def __init__(self, args, patch=None, logger=logging.getLogger('')):
        super(GithubPatchChecker, self).__init__(args=args, patch=patch,
                                                 logger=logger)
        self.github_id = args.gh_id
        self.github_parent_project = args.parent_project
        assert self.github_id is not None
        assert self.github_parent_project is not None
        self.patch = self._fetch_from_github()

    def _get_github_repo_name(self):
        return self.vcs.get_repo_name()

    def _get_github_url(self):
        """
        Gets the correct github URL for the given project.
        """
        repo = self._get_github_repo_name()
        return ("https://github.com/%s/%s/pull/%s.patch" %
                (self.github_parent_project, repo, self.github_id))

    def _fetch_from_github(self):
        """
        Gets a patch file from patchwork and puts it under the base dir.

        :param gh_id: Patchwork patch id.
        """
        patch_url = self._get_github_url()
        patch_dest = os.path.join(self.base_dir,
                                  'github-%s.patch' % self.github_id)
        return utils.download.get_file(patch_url, patch_dest)

    def check(self):
        self.vcs.apply_patch(self.patch)
        passed_check = self.check_modified_files()
        if passed_check:
            self.log.info('Github PR %s check PASS', self.github_id)
            return 0
        else:
            self.log.error('Github PR %s check FAIL', self.github_id)
            return 1


class GithubCommand(Command):
    """
    check github pull requests
    """
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(GithubCommand, self).get_parser(prog_name)
        parser.add_argument('gh_id', type=int,
                            help='GitHub Pull Request ID')
        parser.add_argument('-p', '--parent-project', type=str,
                            help=('Github organization. '
                                  'Default: %(default)s'),
                            default='avocado-framework')
        parser.add_argument('--disable', type=str,
                            help='Disable the pylint errors. Default: %(default)s',
                            default='W,R,C,E1002,E1101,E1103,E1120,F0401,I0011')
        parser.add_argument('--enable', type=str,
                            help=('Enable the pylint errors '
                                  '(takes place after disabled items are '
                                  'processed). Default: %(default)s'),
                            default='W0611')
        parser.add_argument('--pep8-disable', type=str,
                            help='Disable the pep8 errors. Default: %(default)s',
                            default='E501,E265,W601,E402')
        parser.add_argument('--exclude', type=str,
                            help='Quoted string containing paths or '
                                 'patterns to be excluded from '
                                 'checking, comma separated')
        return parser

    def take_action(self, parsed_args):
        checker = GithubPatchChecker(parsed_args, logger=self.log)
        checker.validate()
        return checker.check()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

import logging
import os

from cliff.command import Command

from inspektor.indent import Reindenter
from inspektor.license import LicenseChecker
from inspektor.license import default_license
from inspektor.license import license_mapping
from inspektor.lint import Linter
from inspektor.style import StyleChecker


class CheckAllCommand(Command):
    """
    check indentation, style, lint and (optionally) license headers
    """
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(CheckAllCommand, self).get_parser(prog_name)
        parser.add_argument('path', type=str,
                            help='Path to check (empty for full tree check)',
                            nargs='*',
                            default=None)
        parser.add_argument('--disable-style', type=str,
                            help='Disable the style errors given as arguments. '
                                 'Default: %(default)s',
                            default='E501,E265,W601,E402')
        parser.add_argument('--disable-lint', type=str,
                            help='Disable the pylint errors. Default: %(default)s',
                            default='W,R,C,E1002,E1101,E1103,E1120,F0401,I0011')
        parser.add_argument('--enable-lint', type=str,
                            help=('Enable the pylint errors '
                                  '(takes place after disabled items are '
                                  'processed). Default: %(default)s'),
                            default='W0611')
        parser.add_argument('--fix', action='store_true', default=False,
                            help='Fix any style problems found '
                                 '(needs autopep8 installed)')
        parser.add_argument('--max-line-length', type=int, default=79,
                            help=('set maximum allowed line length. Default: '
                                  '%(default)s'))
        parser.add_argument('--exclude', type=str,
                            help='Quoted string containing paths or '
                                 'patterns to be excluded from '
                                 'checking, comma separated')
        parser.add_argument('--no-license-check', action='store_true', default=False,
                            help='Do not perform license check')
        parser.add_argument('--license', type=str,
                            help=('License type. Supported license types: %s. '
                                  'Default: %s' %
                                  (license_mapping.keys(), default_license)),
                            default="gplv2_later")
        parser.add_argument('--copyright', type=str,
                            help='Copyright string. Ex: "Copyright (c) 2013-2014 FooCorp"',
                            default="")
        parser.add_argument('--author', type=str,
                            help='Author string. Ex: "Author: Brandon Lindon <brandon.lindon@foocorp.com>"',
                            default="")
        return parser

    def take_action(self, parsed_args):
        checked_paths = parsed_args.path
        if not checked_paths:
            checked_paths = [os.getcwd()]

        reindenter = Reindenter(parsed_args, logger=self.log)
        style_checker = StyleChecker(parsed_args, logger=self.log)
        linter = Linter(parsed_args, logger=self.log)
        if not parsed_args.no_license_check:
            self.log.info('License check: (%s)', parsed_args.license)
            license_checker = LicenseChecker(parsed_args, logger=self.log)
        else:
            self.log.info('License check: disabled')
            license_checker = None

        status = linter.check(checked_paths)
        for path in checked_paths:
            status &= reindenter.check(path=path)
            status &= style_checker.check(path=path)
            if license_checker is not None:
                status &= license_checker.check(path=path)

        if status:
            self.log.info('Global check PASS')
            return 0
        else:
            self.log.error('Global check FAIL')
            return 1



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

"""
Implements the base inspektor application.
"""
import sys

from cliff.app import App
from cliff.commandmanager import CommandManager


class InspektorApp(App):

    def __init__(self):
        super(InspektorApp, self).__init__(
            description='Inspektor python code checker and fixer',
            version='0.5.1',
            command_manager=CommandManager('inspektor.app'),
            deferred_help=True,
            )

    def clean_up(self, cmd, result, err):
        if err:
            self.LOG.debug('got an error: %s', err)


def main(argv=sys.argv[1:]):
    inspekt = InspektorApp()
    try:
        return inspekt.run(argv)
    except KeyboardInterrupt:
        inspekt.LOG.error('User requested to interrupt')


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

# pylint: disable=E0611
import sys

from setuptools import setup, find_packages

PROJECT = 'inspektor'

# Change documentation/source/conf.py and inspektor/cli/app.py
VERSION = '0.5.1'

REQUIRES = ['six']
if sys.version_info[:2] == (2, 6):
    REQUIRES += ['astroid==1.2.1', 'pycodestyle>=2.0.0', 'pylint==1.3.1',
                 'logutils>=0.3.3', 'stevedore<=1.10', 'cliff<=1.15.0',
                 'pbr<2.0,>=1.4']
elif sys.version_info[:2] == (2, 7):
    REQUIRES += ['pycodestyle>=2.0.0', 'pylint>=1.3', 'cliff']
elif sys.version_info[0] == 3:
    REQUIRES += ['pycodestyle', 'pylint', 'cliff']

try:
    long_description = open('README.rst', 'rt').read()
except IOError:
    long_description = ''

setup(
    name=PROJECT,
    version=VERSION,

    description='Inspektor python code checker and fixer',
    long_description=long_description,

    author='Lucas Meneghel Rodrigues',
    author_email='lookkas@gmail.com',

    url='https://github.com/avocado-framework/inspektor',
    download_url='https://github.com/avocado-framework/inspektor/tarball/master',

    classifiers=['Development Status :: 3 - Alpha',
                 'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.6',
                 'Intended Audience :: Developers',
                 'Environment :: Console',
                 ],

    platforms=['Any'],

    scripts=[],

    provides=[],
    install_requires=REQUIRES,

    namespace_packages=[],
    packages=find_packages(),
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'inspekt = inspektor.cli.app:main'
        ],
        'inspektor.app': [
            'lint = inspektor.commands.lint:LintCommand',
            'indent = inspektor.commands.indent:IndentCommand',
            'style = inspektor.commands.style:StyleCommand',
            'github = inspektor.commands.github:GithubCommand',
            'license = inspektor.commands.license:LicenseCommand',
            'checkall = inspektor.commands.checkall:CheckAllCommand'
        ],
    },

    zip_safe=False,
)
